# Licensed under a 3-clause BSD style license - see LICENSE.rst

"""
The purpose of this extension is to provide a configuration value that can be
used to disable intersphinx on the command-line without editing conf.py. To use,
you can build documentation with::

    sphinx-build ... -D disable_intersphinx=1

This is used e.g. by astropy-helpers when using the build_docs command.
"""

from sphinx.util.console import bold
from sphinx.util import logging


def disable_intersphinx(app, config=None):
    info = logging.getLogger(__name__).info

    if app.config.disable_intersphinx:
        info(bold('disabling intersphinx...'))
        app.config.intersphinx_mapping.clear()


def setup(app):
    app.connect('config-inited', disable_intersphinx)
    app.add_config_value('disable_intersphinx', 0, True)
    return {'parallel_read_safe': True,
            'parallel_write_safe': True}



# -*- coding: utf-8 -*-
# Licensed under a 3-clause BSD style license - see LICENSE.rst
#
# Astropy shared Sphinx settings.  These settings are shared between
# astropy itself and affiliated packages.
#
# Note that not all possible configuration values are present in this file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

import os
import warnings
from collections import ChainMap
from os import path

try:
    import astropy
except ImportError:
    ASTROPY_INSTALLED = False
else:
    ASTROPY_INSTALLED = True

    from astropy.utils import minversion


# -- General configuration ----------------------------------------------------

# The version check in Sphinx itself can only compare the major and
# minor parts of the version number, not the micro.
needs_sphinx = '4.2'

on_rtd = os.environ.get('READTHEDOCS', None) == 'True'

# Configuration for intersphinx: refer to the Python standard library.
intersphinx_mapping = {
    'python': ('https://docs.python.org/3/',
               (None, 'http://data.astropy.org/intersphinx/python3.inv')),
    'pythonloc': ('http://docs.python.org/',
                  path.abspath(path.join(path.dirname(__file__), '..',
                                         'local', 'python3_local_links.inv'))),
    'numpy': ('https://numpy.org/doc/stable/',
              (None, 'http://data.astropy.org/intersphinx/numpy.inv')),
    'scipy': ('https://docs.scipy.org/doc/scipy/',
              (None, 'http://data.astropy.org/intersphinx/scipy.inv')),
    'matplotlib': ('https://matplotlib.org/stable/',
                   (None, 'http://data.astropy.org/intersphinx/matplotlib.inv')),
    'astropy': ('https://docs.astropy.org/en/stable/', None),
    'h5py': ('https://docs.h5py.org/en/stable/', None),
}

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
exclude_patterns = ['_build']

# Add any paths that contain templates here, relative to this directory.
# templates_path = ['_templates']

# The suffix of source filenames.
source_suffix = '.rst'

# The encoding of source files.
#source_encoding = 'utf-8-sig'

# The master toctree document.
master_doc = 'index'

# The reST default role (used for this markup: `text`) to use for all
# documents. Set to the "smart" one.
default_role = 'obj'

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#language = None

# This is added to the end of RST files - a good place to put substitutions to
# be used globally.
rst_epilog = """
.. _Astropy: https://www.astropy.org
"""

suppress_warnings = ['app.add_directive', ]

# -- NumpyDoc X-Ref ------------------------

# Whether to create cross-references for the parameter types in the
# Parameters, Other Parameters, Returns and Yields sections of the docstring.
# Should be set = True in packages manually! included here as reference.
# numpydoc_xref_param_type = False

# Words not to cross-reference. Most likely, these are common words used in
# parameter type descriptions that may be confused for classes of the same
# name. This can be overwritten or modified in packages and is provided here for
# convenience.
numpydoc_xref_ignore = {"or", "of", "thereof",
                        "default", "optional", "keyword-only",
                        "instance", "type", "class", "subclass", "method"}

# Mappings to fully qualified paths (or correct ReST references) for the
# aliases/shortcuts used when specifying the types of parameters.
# Numpy provides some defaults
# https://github.com/numpy/numpydoc/blob/b352cd7635f2ea7748722f410a31f937d92545cc/numpydoc/xref.py#L62-L94
numpydoc_xref_aliases = {
    # Python terms
    "function": ":term:`python:function`",
    "iterator": ":term:`python:iterator`",
    "mapping": ":term:`python:mapping`",
}

# Aliases to Astropy's glossary. In packages these can be turned on with
# ``numpydoc_xref_aliases.update(numpydoc_xref_aliases_astropy_glossary)``
# (if astropy is in the intersphinx mapping).
numpydoc_xref_aliases_astropy_glossary = {}  # works even if no Astropy
if ASTROPY_INSTALLED and minversion(astropy, "4.3"):
    numpydoc_xref_aliases_astropy_glossary = {
        # general
        "-like": ":term:`astropy:-like`",
        # coordinates
        "angle-like": ":term:`astropy:angle-like`",
        "coordinate-like": ":term:`astropy:coordinate-like`",
        "frame-like": ":term:`astropy:frame-like`",
        # units
        "unit-like": ":term:`astropy:unit-like`",
        "quantity-like": ":term:`astropy:quantity-like`",
        # table
        "table-like": ":term:`astropy:table-like`",
        # time
        "time-like": ":term:`astropy:time-like`",
    }

# Aliases to Astropy's physical types. In packages these can be turned on with
# ``numpydoc_xref_aliases.update(numpydoc_xref_aliases_astropy_physical_type)``
# (if astropy is in the intersphinx mapping).
numpydoc_xref_aliases_astropy_physical_type = {}  # works even if no astropy
if ASTROPY_INSTALLED and minversion(astropy, "4.3"):

    from astropy.units.physical import _name_physical_mapping
    for ptype in _name_physical_mapping.keys():
        val = f":ref:`:ref: '{ptype}' <astropy:{ptype}>`"   # <= intersphinxed
        numpydoc_xref_aliases_astropy_physical_type[f"'{ptype}'"] = val

    del ptype, val, _name_physical_mapping  # cleanup namespace


# Convenient collection of all of astropy's options for numpydoc xref.
# In packages all the astropy additions can be turned on with
# ``numpydoc_xref_aliases.update(numpydoc_xref_astropy_aliases)``
# (if astropy is in the intersphinx mapping).
numpydoc_xref_astropy_aliases = ChainMap(  # important at the top
    numpydoc_xref_aliases_astropy_glossary,
    numpydoc_xref_aliases_astropy_physical_type
)

# -- Project information ------------------------------------------------------

# There are two options for replacing |today|: either, you set today to some
# non-false value, then it is used:
#today = ''
# Else, today_fmt is used as the format for a strftime call.
#today_fmt = '%B %d, %Y'

# If true, '()' will be appended to :func: etc. cross-reference text.
#add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
#add_module_names = True

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
#show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
#pygments_style = 'sphinx'

# A list of ignored prefixes for module index sorting.
#modindex_common_prefix = []


# -- Settings for extensions and extension options ----------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.coverage',
    'sphinx.ext.inheritance_diagram',
    'sphinx.ext.intersphinx',
    'sphinx.ext.mathjax',
    'sphinx.ext.todo',
    'sphinx.ext.viewcode',
    'sphinxcontrib.jquery',
    'numpydoc',
    'sphinx_copybutton',
    'pytest_doctestplus.sphinx.doctestplus',
    'sphinx_astropy.ext.changelog_links',
    'sphinx_astropy.ext.generate_config',
    'sphinx_astropy.ext.intersphinx_toggle',
    'sphinx_astropy.ext.missing_static',
    'sphinx_automodapi.automodapi',
    'sphinx_automodapi.smart_resolver',
]

try:
    import matplotlib.sphinxext.plot_directive  # noqa: F401
    extensions += ['matplotlib.sphinxext.plot_directive']
# AttributeError is checked here in case matplotlib is installed but
# Sphinx isn't.  Note that this module is imported by the config file
# generator, even if we're not building the docs.
except (ImportError, AttributeError):
    warnings.warn(
        "matplotlib's plot_directive could not be imported. " +
        "Inline plots will not be included in the output")

# Don't show summaries of the members in each class along with the
# class' docstring
numpydoc_show_class_members = False

autosummary_generate = True

automodapi_toctreedirnm = 'api'

# Class documentation should contain *both* the class docstring and
# the __init__ docstring
autoclass_content = "both"

# Render inheritance diagrams in SVG
graphviz_output_format = "svg"

graphviz_dot_args = [
    '-Nfontsize=10',
    '-Nfontname=Helvetica Neue, Helvetica, Arial, sans-serif',
    '-Efontsize=10',
    '-Efontname=Helvetica Neue, Helvetica, Arial, sans-serif',
    '-Gfontsize=10',
    '-Gfontname=Helvetica Neue, Helvetica, Arial, sans-serif',
]

# For sphinx-copybutton
copybutton_prompt_text = r">>> |\.\.\. |\$ |In \[\d*\]: | {2,5}\.\.\.: | {5,8}: "
copybutton_prompt_is_regexp = True

# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
html_theme = 'pydata_sphinx_theme'

# The name of an image file (within the static path) to use as favicon of the
# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.

# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
# using the given strftime format.
html_last_updated_fmt = '%d %b %Y'

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
html_theme_options = {
    "collapse_navigation": True,
    "icon_links": [],
    "navigation_depth": 2,
    "show_nav_level": 2,
    "use_edit_page_button": False
}

# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
#html_title = None

# A shorter title for the navigation bar.  Default is the same as html_title.
#html_short_title = None

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
#html_use_smartypants = True

# Additional templates that should be rendered to pages, maps page names to
# template names.
#html_additional_pages = {}

# If false, no module index is generated.
#html_domain_indices = True

# If false, no index is generated.
#html_use_index = True

# If true, the index is split into individual pages for each letter.
#html_split_index = False

# If true, links to the reST sources are added to the pages.
#html_show_sourcelink = True

# If true, "Created using Sphinx" is shown in the HTML footer. Default is True.
#html_show_sphinx = True

# If true, "(C) Copyright ..." is shown in the HTML footer. Default is True.
#html_show_copyright = True

# If true, an OpenSearch description file will be output, and all pages will
# contain a <link> tag referring to it.  The value of this option must be the
# base URL from which the finished HTML is served.
#html_use_opensearch = ''

# This is the file name suffix for HTML files (e.g. ".xhtml").
#html_file_suffix = None

# -- Options for LaTeX output ------------------------------------------------

# The paper size ('letter' or 'a4').
#latex_paper_size = 'letter'

# The font size ('10pt', '11pt' or '12pt').
#latex_font_size = '10pt'

# For "manual" documents, if this is true, then toplevel headings are parts,
# not chapters.
latex_toplevel_sectioning = 'part'

# If true, show page references after internal links.
#latex_show_pagerefs = False

# If true, show URL addresses after external links.
#latex_show_urls = False

latex_elements = {}

# Additional stuff for the LaTeX preamble.
latex_elements['preamble'] = r"""
% Use a more modern-looking monospace font
\usepackage{inconsolata}

% The enumitem package provides unlimited nesting of lists and enums.
% Sphinx may use this in the future, in which case this can be removed.
% See https://bitbucket.org/birkenfeld/sphinx/issue/777/latex-output-too-deeply-nested
\usepackage{enumitem}
\setlistdepth{15}

% In the parameters section, place a newline after the Parameters
% header.  (This is stolen directly from Numpy's conf.py, since it
% affects Numpy-style docstrings).
\usepackage{expdlist}
\let\latexdescription=\description
\def\description{\latexdescription{}{} \breaklabel}

% Support the superscript Unicode numbers used by the "unicode" units
% formatter
\DeclareUnicodeCharacter{2070}{\ensuremath{^0}}
\DeclareUnicodeCharacter{00B9}{\ensuremath{^1}}
\DeclareUnicodeCharacter{00B2}{\ensuremath{^2}}
\DeclareUnicodeCharacter{00B3}{\ensuremath{^3}}
\DeclareUnicodeCharacter{2074}{\ensuremath{^4}}
\DeclareUnicodeCharacter{2075}{\ensuremath{^5}}
\DeclareUnicodeCharacter{2076}{\ensuremath{^6}}
\DeclareUnicodeCharacter{2077}{\ensuremath{^7}}
\DeclareUnicodeCharacter{2078}{\ensuremath{^8}}
\DeclareUnicodeCharacter{2079}{\ensuremath{^9}}
\DeclareUnicodeCharacter{207B}{\ensuremath{^-}}
\DeclareUnicodeCharacter{00B0}{\ensuremath{^{\circ}}}
\DeclareUnicodeCharacter{2032}{\ensuremath{^{\prime}}}
\DeclareUnicodeCharacter{2033}{\ensuremath{^{\prime\prime}}}
"""

# Documents to append as an appendix to all manuals.
#latex_appendices = []

# If false, no module index is generated.
#latex_domain_indices = True

# The name of an image file (relative to this directory) to place at the top of
# the title page.
#latex_logo = None

# -- Options for the linkcheck builder ----------------------------------------

# A timeout value, in seconds, for the linkcheck builder
linkcheck_timeout = 60



# This directory contains the default Sphinx configuration for the core Astropy
# package and other packages that want to use the same configuration. Rather
# than store the configuration in a single conf.py file, we store it in v?.py
# files and import the latest one into sphinx_astropy.conf so that packages can
# choose to use either to do:
#
#     from sphinx_astropy.conf import *
#
# or:
#
#     from sphinx_astropy.conf.v1 import *
#
# with the latter being the option to use for stability. The idea is that
# we can still make small changes (mainly fixing bugs) to v1.py, but if we
# make any big changes in future, we can create a new version that packages
# can choose to opt-in to. To create a new default configuration, create a
# v2.py file (either starting from a copy of v1.py or starting from
# scratch), and change the import below to 'from .v2 import *'.

# TODO: Switch default to v2
from .v1 import *  # noqa: F401, F403



import io

from docutils import nodes
from docutils.parsers.rst import Directive


class GenerateConfig(Directive):
    """
    Directive to generate the configuration file for a package and
    include it in the documentation as a literal code block. This relies
    on the ``generate_config`` function, added in Astropy 4.1.

    Example::

        .. generate_config:: astropy

    """

    has_content = False
    required_arguments = 1

    def run(self):
        from astropy.config import generate_config
        buf = io.StringIO()
        generate_config(pkgname=self.arguments[0], filename=buf)
        text = buf.getvalue()
        node = nodes.literal_block(text, text)
        return [node]


def setup(app):
    app.add_directive("generate_config", GenerateConfig)

    return {
        'version': '0.1',
        'parallel_read_safe': True,
        'parallel_write_safe': True,
    }



import pytest

pytest.importorskip("pydata_sphinx_theme")

from sphinx.cmd.build import build_main  # noqa: E402

BASIC_CONF = """
from sphinx_astropy.conf.v2 import *
suppress_warnings = ['app.add_directive', 'app.add_node', 'app.add_role']
"""

BASIC_INDEX = """
Title
=====

Just a test
"""


def generate_files(tmp_path):
    f1 = tmp_path / "conf.py"
    f1.write_text(BASIC_CONF)

    f2 = tmp_path / "index.rst"
    f2.write_text(BASIC_INDEX)


def test_conf(tmp_path):

    # Make sure the docs build with the v2 sphinx-astropy configuration

    generate_files(tmp_path)

    src_dir = str(tmp_path)
    html_dir = str(tmp_path / "html")

    status = build_main(argv=['-W', '-b', 'html', src_dir, html_dir])

    assert status == 0


def test_intersphinx_toggle(tmp_path, capsys):

    # Test the sphinx_astropy.ext.intersphinx_toggle extension

    generate_files(tmp_path)

    src_dir = str(tmp_path)
    html_dir = str(tmp_path / "html")

    status = build_main(argv=['-W', '-b', 'html', src_dir, html_dir, '-D',
                              'disable_intersphinx=1'])

    assert status == 0

    captured = capsys.readouterr()
    assert 'disabling intersphinx' in captured.out
    assert 'loading intersphinx' not in captured.out



#!/usr/bin/env python

# Licensed under a 3-clause BSD style license - see LICENSE.rst

import os
from setuptools import setup

setup(use_scm_version={'write_to': os.path.join('sphinx_astropy', 'version.py')})
